/*
 * This file handles all of the user interface issues for
 * the MediaView example player.
 */
#include <stdlib.h>
#include <ctype.h>
#include <memory.h>	 
#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#include <string.h>
#include <stdarg.h>
#include "ctl3d.h"

/* MediaView Include Files */
#include <medv14.h>

/* Application Include Files */
#include "topic.h"
#include "player.h"
#include "pane.h"
#include "annotate.h"
#include "resrc1.h"
#include "proto.h"

/*
 * This applicaton keeps around 1 TOPIC (window with scrolling and
 * non-scrolling regions), and a single PANE containing the contents
 * page of the current title.
 */
LPTOPIC lpTopic = {0};     
HWND hTOC = 0;

/*
 * Global Variables.
 */
int bShowHotspots = FALSE;	/* hotspots currently visible? */
int iKerningBoundary = 0;	/* kerning boundry for text */
HANDLE hAccTable = 0; 		/* handle to accelerator table */
HWND hMainWnd = 0;			/* handle to main window */
HWND hFP = 0;				/* face plate handle */
HWND hActiveTopic = 0;		/* Which Topic has the "focus" */
int numOpenTitles = 0;		/* number of open titles */
HTITLE *OpenTitles = 0;		/* table of open title handles */
char gErrBuff[512] = {0};	/* printer error buffer */
HWND hHist = 0;				/* handle of history list window */
HWND hSearch = 0;			/* handle of the search results window */
HANDLE hTopicList = 0;		/* search results topic list */
HANDLE hHighlights = 0;		/* search results highlights list */
int showHits = FALSE;		/* show the search highlights? */
HBRUSH hHitBrush = 0;		/* current brush for search hilights */
HANDLE hWordWheel = 0;		/* the current open word wheel */
int iIndex = 0;				/* the currently open index group number */
LPGROUPUI lpGroups = 0;		/* group list for the title */
LPGROUPUI lpKeyIndex = 0;	/* list of keyword index groups */
LPTOPIC lpSecondary = {0};	/* the secondary window */
HFILE hfAnnotation = 0;		/* Annotation file handle */
HINSTANCE ghInstance = 0;	/* global application instance */
int gStopSearch = FALSE;	/* Variable to indicate a search abort */
RECT rcSecondary = {160, 120, 480, 360};	/* Remember the secondary window position */

char szWindowTitle[] = "MediaView 1.4 Example Player";

LPSTR szInitialTopic = 0;

/* list of menu items that are grayed on entry, then enabled at file open */
int MenuItems[] =	{
			        ID_FILE_PRINTTOPIC,
        			ID_FILE_PRINTMARKED,
        			ID_FILE_PRINTALLTOPICS,
        			ID_EDIT_COPYTOPIC,
        			ID_EDIT_COPYSELECTION,
        			ID_EDIT_SEARCH,
        			ID_EDIT_CHARACTERSTYLES,
        			ID_BOOKMARK_DEFINE,
        			ID_TOPIC_INDEX,
	       			ID_TOPIC_MARK,
        			ID_VIEW_SEARCHRESULTS,
        			ID_VIEW_EMBEDDEDWINDOW,
        			ID_VIEW_HIGHLIGHTS,
					ID_VIEW_HOTSPOTS,
        			ID_OPTIONS_TEXTCOLOR,
        			ID_OPTIONS_HITCOLOR,
        			ID_OPTIONS_MAGNIFYTEXT,
					ID_OPTIONS_KERNING,
					ID_OPTIONS_SELECTCOLOR,
					};
/* variables for GetOpenFilename commmon dialog */
OPENFILENAME ofn = {0};
char szFilterSpec [128] =		/* file type filters */
             "MediaView 1.4 (*.M14)\0*.M14\0All Files (*.*)\0*.*\0";
char szFileName[_MAX_PATH] = {0};
char szFileTitle[_MAX_PATH] = {0};

/* variables for the Print common dialog */
PRINTDLG pd = {0};

/* variables for ChooseColor common dialog */
CHOOSECOLOR cc = {0};
COLORREF aclrCust[16] = {0};

/* cursors for different hotspot types */
HCURSOR curT = 0;
HCURSOR curB = 0;
HCURSOR curE = 0;
HCURSOR curU = 0;
HCURSOR curArrow = 0;
HCURSOR curIbeam = 0;
HCURSOR curWait = 0;

/* main Window Class */
#define MVAPPCLASS "MVAppClass"

/****************************************************************************
 **     FUNCTION: WinMain                                                  **
 **     PURPOSE: initialize and call do main processing loop               **
 **     COMMENTS:                                                          **
 ****************************************************************************/
int WINAPI WinMain(
	HINSTANCE hInstance,
	HINSTANCE hPrevInstance,
	LPSTR lpCmdLine,
	int nCmdShow)
	{
	MSG msg;  
	int ret = TRUE;
	int argc;
	LPSTR argv[10];
	LPSTR szFilename;

	ghInstance = hInstance;

	/* Enable the 3-D controls */
	Ctl3dRegister(hInstance);
	Ctl3dAutoSubclass(hInstance);
	argc = UI_ParseCmdLine(lpCmdLine, argv, DIMENSION(argv));
	/* Initialize the MV libraries */
	MVSetInstance(hInstance);

	if (!hPrevInstance)
		if (!InitApplication(hInstance))
			{
			ret = FALSE;
			goto exit;
			}
		
	if (!InitInstance(hInstance, nCmdShow))
			{
			ret = FALSE;
			goto exit;
			}

	/* do initial sizing of faceplate and source window */
	UI_ResizeFacePlate(hMainWnd, hFP, 0, 0);

	/* everything is set up, show the main window */
	ShowWindow(hMainWnd, SW_SHOW);

	/* was there a filename on the command line? */
	if (argc > 0)
		{
			szFilename = argv[0];

		if (UI_OpenFile(szFilename, hInstance, 0, szInitialTopic) != 0)
			{
			/* enable the button bar */
			UI_EnableControls(hFP, TRUE,
								IDC_INDEX,
								IDC_SEARCH,
								0);

			/* enable the portions of the menu that need an open file */
			UI_EnableMenuItems(hMainWnd, MenuItems, DIMENSION(MenuItems), MF_ENABLED|MF_BYCOMMAND);

			/*
			 * Test of Baggage file system APIs. This code does not
			 * do anything meaningful, it just demonstrates the use of
			 * these APIs.
			 */
			ret = UI_BaggageTest(OpenTitles[0]);
			}
		}

	/* the main message loop */
	while (GetMessage(&msg, 0, 0, 0)) 
		{ 
		/* modeless dialog boxes need custom dispatching */
		if (hSearch && IsDialogMessage(hSearch, &msg))
			continue;
		if (hHist && IsDialogMessage(hHist, &msg))
			continue;

		 /* otherwise, dispatch to the main message pump */
		else 
		if (!TranslateAccelerator(hMainWnd, hAccTable, &msg))
			{
			TranslateMessage(&msg);
			DispatchMessage(&msg); 
			}
		}
	ret = msg.wParam;
	exit:

	/* and terminate the MV libraries */
	MVTerminate(hInstance);

	/* clean up the 3 -D controls */
	Ctl3dUnregister(hInstance);
	return(ret);
	}


/****************************************************************************
 **     FUNCTION: InitApplication                                          **
 **     PURPOSE: initialize window data and registers window class         **
 **     COMMENTS:                                                          **
 ****************************************************************************/
BOOL InitApplication(HANDLE hInstance)
	{
	WNDCLASS  wc;

	wc.style = 0;
	wc.lpfnWndProc = MainWndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = LoadIcon (hInstance, "myicon");
	wc.hCursor = LoadCursor(0, IDC_ARROW);
	wc.hbrBackground = GetStockObject(LTGRAY_BRUSH); 
	wc.lpszMenuName =  "MVAppMenu";
	wc.lpszClassName = MVAPPCLASS;
	if (RegisterClass(&wc) == 0)
		return(FALSE);

	if (!Topic_Register(hInstance))
		return(FALSE);
		
	if (!Pane_Register(hInstance))
		return(FALSE);
	return(TRUE);
	}

/****************************************************************************
 **     FUNCTION: InitInstance                                             **
 **     PURPOSE: create the windows and create the button bar              **
 **     COMMENTS:                                                          **
 ****************************************************************************/
BOOL InitInstance(HANDLE hInstance, int nCmdShow)
	{
	int i;
	
	hAccTable = LoadAccelerators(hInstance, "MVAppAccelerators");

	/* create the main application window */
	hMainWnd = CreateWindowEx(
#ifdef WIN32
			WS_EX_OVERLAPPEDWINDOW,
#else
			0,
#endif
			MVAPPCLASS,
			szWindowTitle,
    	    WS_OVERLAPPEDWINDOW,
			0,
			0,
			640,
			480,
			0,
			0,
			hInstance,
			0 );

	if (!hMainWnd)
		return (FALSE);      
		
	ShowWindow(hMainWnd, SW_HIDE);
	/* set the opeing topic of the right hand window */
	szInitialTopic = "opening";

	/* grey out the portions of the menu that need an open file */
	UI_EnableMenuItems(hMainWnd, MenuItems, DIMENSION(MenuItems), MF_GRAYED|MF_BYCOMMAND);
	UpdateWindow(hMainWnd);

	/* manifest the button bar */
	if ((hFP = CreateDialog(hInstance, "FacePlateDlg", hMainWnd, (DLGPROC)FacePlateDlgProc)) == 0)
		return(FALSE);

	/* fill in non-variant fields of OPENFILENAME struct. */
	ofn.lStructSize       = sizeof(OPENFILENAME);
	ofn.hwndOwner	  = hMainWnd;
	ofn.lpstrFilter	  = szFilterSpec;
	ofn.lpstrCustomFilter = 0;
	ofn.nMaxCustFilter	  = 0;
	ofn.nFilterIndex	  = 1;
	ofn.lpstrFile         = szFileName;
	ofn.nMaxFile	  		 = sizeof(szFileName);
	ofn.lpstrInitialDir   = 0;
	ofn.lpstrFileTitle    = szFileTitle;
	ofn.nMaxFileTitle     = sizeof(szFileTitle);
	ofn.lpstrTitle        = 0;
	ofn.lpstrDefExt       = "M14";
	ofn.Flags             = OFN_HIDEREADONLY;

	/* initialize the PRINTDLG structure */
	_fmemset(&pd, 0, sizeof(PRINTDLG));
	pd.lStructSize = sizeof(PRINTDLG);
	pd.hwndOwner = hMainWnd;
	pd.Flags = PD_RETURNDC | PD_NOSELECTION | PD_NOPAGENUMS;
	/* initialize the CHOOSECOLOR structure */
	for (i = 0; i < 16; i++)
		{
		/* Set the custom color controls to white. */
		aclrCust[i] = RGB(255, 255, 255);
		}
	cc.lStructSize = sizeof(CHOOSECOLOR);
	cc.hwndOwner = hMainWnd;
	cc.rgbResult = RGB(0, 0, 0);
	cc.lpCustColors = aclrCust;
	cc.Flags = CC_PREVENTFULLOPEN;
	UI_InitCursors(hInstance);
	/* default to a white brush for painting Search highlights */
	hHitBrush = CreateSolidBrush(RGB(255,255,255));
	return (TRUE);
	}

/****************************************************************************
 **     FUNCTION: MainWndProc                                              **
 **     PURPOSE: Handle top level messages to the main window              **
 **     COMMENTS:                                                          **
 ****************************************************************************/
long WINAPI MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LONG lParam)
	{
	UINT cmd;
	ERR err = 0;
	LPSTR szFilename;
	int state;
	VA va;
	long scroll;
	HTITLE hTitle;
	int oldB;
#ifdef WIN32
	HINSTANCE hInst = (HINSTANCE)GetWindowLong(hWnd, GWL_HINSTANCE);
#else
	HINSTANCE hInst = (HINSTANCE)GetWindowWord(hWnd, GWW_HINSTANCE);
#endif	//WIN32

	switch (message) 
		{
#ifdef WIN32
		/* ALT-TAB to another application on NT loses the mouse
		 * capture, so re-capture it when re-activated.
		 */
		case WM_ACTIVATEAPP:
			{
			extern HWND hCaptureWnd;
			if (wParam == TRUE && hCaptureWnd)
				SetCapture(hCaptureWnd);
				break;
			}
#endif
		case WM_KEYDOWN:
			/* send keystrokes to the most recently clicked in region */
			if (hTOC && !hActiveTopic)
				
				SendMessage(hTOC, message, wParam, lParam);
			else
				if (hActiveTopic)
					SendMessage(GetTopicPtr(hActiveTopic)->hTopic, message, wParam, lParam);
				break;

		case WM_SIZE:
			UI_ResizeFacePlate(hWnd, hFP, (lpTopic?lpTopic->hTopic:0), hTOC);
			break;

		case UWM_TOPIC_DESTROY:
			/* was the secondary window destroyed? */
			if (lpSecondary && (HWND)lParam == lpSecondary->hTopic)
				lpSecondary = 0;
			SetFocus(hMainWnd);
			break;

		case UWM_PANE_FOCUS:
			break;
		case UWM_TOPIC_FOCUS:
			/* Clear selection from previous active Topic? */
			hWnd = (HWND)lParam;
			if (hActiveTopic && hActiveTopic != hWnd)
				{
				if (hWnd == lpTopic->hTopic && lpSecondary)
					Topic_ClearSelection(lpSecondary);
				else
					Topic_ClearSelection(lpTopic);
				}
			hActiveTopic = hWnd;
			break;

		case WM_COMMAND:

			cmd = GET_WM_COMMAND_ID(wParam, lParam);
			switch (cmd) 
				{
				case ID_FILE_OPEN:
					/* call the common dialog to get a file name */
					if (!GetOpenFileName ((LPOPENFILENAME)&ofn))
						break;
					szFilename = ofn.lpstrFile;
					
					/* if there is already a topic, save it to the history list */
					if (lpTopic)
						MV_AddToHistory(Topic_ValidMV(lpTopic));
					if (UI_OpenFile(szFilename, hInst, 0, szInitialTopic) != 0)
				 		{
						/* enable the button bar */
						UI_EnableControls(hFP, TRUE,
								IDC_INDEX,
								IDC_SEARCH,
								0);

						/* enable the portions of the menu that need an open file */
						UI_EnableMenuItems(hMainWnd, MenuItems, DIMENSION(MenuItems), MF_ENABLED|MF_BYCOMMAND);
						}
					else
						{
						/* disable buttons that need an open file */
						UI_EnableControls(hFP, FALSE,
								IDC_INDEX,
								IDC_SEARCH,
								0);

						/* disable the portions of the menu that need an open file */
						UI_EnableMenuItems(hMainWnd, MenuItems, DIMENSION(MenuItems), MF_GRAYED|MF_BYCOMMAND);
						}
					break;
				case ID_FILE_PRINTTOPIC:
					if (PrintDlg(&pd) == 0)
						break;
					if (MV_PrintCurrentTopic(Topic_ValidMV(lpTopic), pd.hDC) != 0)
						ErrorMsg(gErrBuff, "Print Error");
					break;
				case ID_FILE_PRINTMARKED:
					if (PrintDlg(&pd) == 0)
						break;
					if (MV_PrintMarkList(Topic_ValidMV(lpTopic), pd.hDC) != 0)
						ErrorMsg(gErrBuff, "Print Error");
					break;
				case ID_FILE_PRINTALLTOPICS:
					if (PrintDlg(&pd) == 0)
						break;
					if (MV_PrintAll(Topic_ValidMV(lpTopic), pd.hDC) != 0)
						ErrorMsg(gErrBuff, "Print Error");
					break;
				case ID_TOPIC_MARK:
					DialogBox(hInst, "MarkDlg", hWnd, MarkDlgProc);
					break;
				case ID_FILE_EXIT:
					DestroyWindow(hWnd);
					break;
				case ID_EDIT_COPYTOPIC:
					Topic_CopyTopic(hActiveTopic ? GetTopicPtr(hActiveTopic) : lpTopic);
					break;
				case ID_EDIT_COPYSELECTION:
					Topic_CopySelection(hActiveTopic ? GetTopicPtr(hActiveTopic) : lpTopic);
					break;
				case ID_EDIT_SEARCH:
					DialogBox(hInst, "SearchDlg", hWnd, SearchDlgProc);
					break;
				case ID_TOPIC_INDEX:
					DialogBox(hInst, "IndexDlg", hWnd, IndexDlgProc);
					break;
				case ID_TOPIC_HISTORY:
					if (hHist == 0)
						hHist = CreateDialog(hInst, "HistoryDlg", hWnd, HistoryDlgProc);
					BringWindowToTop(hHist);
					break;
				case ID_TOPIC_BACK:
					oldB = MV_Back(&hTitle, &va, &scroll);
					MV_AddToHistory(Topic_ValidMV(lpTopic));
					MV_BackAdjust1(oldB);
					MV_ChangeAddress(hTitle, lpTopic, va, scroll);
					MV_BackAdjust2();
					break;
				case ID_TOPIC_NEXT:
					MV_NextPrev(lpTopic, IDC_NEXT);
					break;
				case ID_TOPIC_PREV:
					MV_NextPrev(lpTopic, IDC_PREV);
					break;
				case ID_VIEW_NEXTHIGHLIGHT:
					MV_ScrollToNextHighlight(GetPaneMV(lpTopic->hSR));
					break;
				case ID_VIEW_HIGHLIGHTS:
					showHits = UI_ToggleMenuItem(hWnd, ID_VIEW_HIGHLIGHTS);
					
					/* display the search hits (on or off) */
					if (fMVHasNSR(GetPaneMV(lpTopic->hNSR)))
						MV_UpdateSearchHits(GetPaneMV(lpTopic->hNSR));
					if (fMVHasSR(GetPaneMV(lpTopic->hSR)))
						MV_UpdateSearchHits(GetPaneMV(lpTopic->hSR));
					break;
				case ID_VIEW_SEARCHRESULTS:
					state = UI_ToggleMenuItem(hWnd, ID_VIEW_SEARCHRESULTS);
					if (state == MF_CHECKED)
						{
						/* Create the Search Results display window */
						CreateDialog(hInst, "SearchResultsDlg", hMainWnd, (DLGPROC)SearchResultsDlgProc);
						BringWindowToTop(hSearch);
						SetFocus(GetDlgItem(hSearch, IDC_LIST1));
						}
					else
						{
						/* close the Search Results window */
						SendMessage(hSearch, WM_CLOSE, 0, 0);
						}
					break;
				case ID_VIEW_HOTSPOTS:
					bShowHotspots = UI_ToggleMenuItem(hWnd, ID_VIEW_HOTSPOTS);
					Topic_HotspotHighlights(lpTopic, bShowHotspots == MF_CHECKED);
					if (hTOC)
						fMVHighlightHotspots(GetPaneMV(hTOC), bShowHotspots == MF_CHECKED, &err);

					break;
				case ID_OPTIONS_HITCOLOR:
					if (ChooseColor(&cc))
						{
						DeleteObject(hHitBrush);
						hHitBrush = CreateSolidBrush(cc.rgbResult);

						Topic_HighlightLook(lpTopic, hHitBrush);
						}
					break;    
	/*******************************************************/
	/******** TUTORIAL: Insert Lesson 14 code here. ********/
	/*******************************************************/
					case ID_OPTIONS_SELECTCOLOR:
					if (ChooseColor(&cc))
					   Topic_SelectColor(lpTopic, cc.rgbResult);
					break;
	/*******************************************************/
	/******** TUTORIAL: Insert Lesson 14 code here. ********/
	/*******************************************************/
				case ID_HELP_ABOUT:
					DialogBox(hInst, "AboutDlg", hWnd, AboutDlgProc);
					break;
	/*******************************************************/
	/******** TUTORIAL: Insert Lesson 14 code here. ********/
	/*******************************************************/
				case ID_BOOKMARK_DEFINE:
					DialogBox(hInst, "BookmarkDlg", hWnd, BookmarkDlgProc);
					break;
				default:
					/* these menu items are added dynamically */
					if (wParam >= ID_BOOKMARKBASE && wParam < ID_BOOKMARKBASE + NUMBOOKMARKS)
						{
						MV_GoToFromBookmark(wParam - ID_BOOKMARKBASE, &hTitle, &va, &scroll);
						MV_AddToHistory(Topic_ValidMV(lpTopic));
						MV_ChangeAddress(hTitle, lpTopic, va, scroll);
						}
					break;
				} 
			break;

		case WM_DESTROY:
			/* close down any open title */
			if (lpTopic && lpTopic->isOpen)
				UI_CloseFile();
			/* any other resources to free? */
			MV_CleanupHistory();
			MV_FreePrintMarks();
			UI_CloseTitles();
			UI_DeleteCursors();  
			if ( hFP )
				DestroyWindow(hFP);
			if (hHitBrush)
				DeleteObject(hHitBrush);
			PostQuitMessage(0);            
			break;

		default:
			/* otherwise ... */
			return (DefWindowProc(hWnd, message, wParam, lParam));
		}
	return (0);
	}

/****************************************************************************
 **     FUNCTION: UI_ResizeFacePlate                                       **
 **     PURPOSE: Respond to size changes in the parent and resize the      **
 **       FacePlate.  This includes resizing the Topic display window.      **
 **     COMMENTS:                                                          **
 ****************************************************************************/
void UI_ResizeFacePlate(HWND hMain, HWND hFP, HWND hTopic, HWND hTOC)
	{
	RECT rect;
	int x, y;    
	int xT = 0;
	int xV = 0;
	
	/* size the whole face plate */
	GetClientRect(hMain, &rect);
	x = rect.right - rect.left;
	y = rect.bottom - rect.top;
	MoveWindow(hFP, 0, 0, x, y, TRUE);

	/* size the Topic display frame */
	#define FRAME 10
	#define BORDER 2
	GetClientRect(GetDlgItem(hFP, IDC_INDEX), &rect);
	UI_ConvertCoordinates(hFP, GetDlgItem(hFP, IDC_INDEX), &rect);
	xT = (x - 3*FRAME) / 3;
	xV = (x - 3*FRAME) - xT;

	/* TOC sized with a border inside the left faceplate window */
	MoveWindow(GetDlgItem(hFP, IDC_TOC), FRAME, rect.bottom + FRAME, xT, y - 2*FRAME - rect.bottom, TRUE);
	if (hTOC)
		MoveWindow(hTOC, FRAME+BORDER, rect.bottom+BORDER+FRAME, xT-2*BORDER, y - 2*FRAME - rect.bottom - 2*BORDER, TRUE);

	/* Topic */
	MoveWindow(GetDlgItem(hFP, IDC_STATIC), 2*FRAME+xT, rect.bottom + FRAME, xV, y - 2*FRAME - rect.bottom, TRUE);

	/* size the Topic with a 2 pixel border inside the right faceplate window */
	if (hTopic)
		UI_SizeTopicInWindow(hFP, IDC_STATIC, hTopic, BORDER);

	InvalidateRect(hFP, 0, TRUE);
	}

/****************************************************************************
 **     FUNCTION: UI_ConvertCoordinates                                    **
 **     PURPOSE: Convert the RECT coordiantes from one origin to another   **
 **     COMMENTS:                                                          **
 ****************************************************************************/
 void UI_ConvertCoordinates(HWND hTo, HWND hFrom, LPRECT lpR)
 	{
	POINT pt;

	pt.x = lpR->left;
	pt.y = lpR->top;
	ClientToScreen(hFrom, &pt);
	ScreenToClient(hTo, &pt);
	lpR->left = pt.x;
	lpR->top = pt.y;
	pt.x = lpR->right;
	pt.y = lpR->bottom;
	ClientToScreen(hFrom, &pt);
	ScreenToClient(hTo, &pt);
	lpR->right = pt.x;
	lpR->bottom = pt.y;
	}

/****************************************************************************
 **     FUNCTION: UI_SizeTopicInWindow                                     **
 **     PURPOSE: Size the Topic into a particular control in a window.     **
 **     COMMENTS:                                                          **
 **       The border allows the Topic to have a margin inside the control. **
 ****************************************************************************/
void UI_SizeTopicInWindow(HWND hWnd, int control, HWND hTopic, int border)
	{ 
	RECT rect;
	POINT pt;

	/* now size the Topic itself within the display frame */
	GetWindowRect(GetDlgItem(hWnd, control), &rect);
	pt.x = rect.left;
	pt.y = rect.top;
	ScreenToClient(hWnd, &pt);

	/* these force the layout and then the painting of the Topic */
	MoveWindow(hTopic, pt.x+border, pt.y+border, rect.right - rect.left-2*border, rect.bottom - rect.top-2*border, TRUE);
	InvalidateRect(hTopic, 0, TRUE);
	}

/****************************************************************************
 **     FUNCTION: FacePlateDlgProc                                         **
 **     PURPOSE: Dispatch the buttons from the button bar                  **
 **     COMMENTS:                                                          **
 ****************************************************************************/
BOOL WINAPI FacePlateDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{   
	UINT	cmd;
	HWND hWnd;

	switch (message) 
		{
		case WM_INITDIALOG:
			UI_EnableControls(hDlg, FALSE,
											IDC_NEXTHIGHLIGHT,
											IDC_INDEX,
											IDC_BACK,
											IDC_HISTORY,
											IDC_NEXT,
											IDC_PREV,
											IDC_SEARCH,
											0);
			break;

		case WM_COMMAND:
			cmd = GET_WM_COMMAND_ID(wParam, lParam);
			switch (cmd)
				{
				case IDC_NEXTHIGHLIGHT:
					SendMessage(hMainWnd, WM_COMMAND, ID_VIEW_NEXTHIGHLIGHT, 0);
					break;
				case IDC_INDEX:
					SendMessage(hMainWnd, WM_COMMAND, ID_TOPIC_INDEX, 0);
					break;
				case IDC_BACK:
					SendMessage(hMainWnd, WM_COMMAND, ID_TOPIC_BACK, 0);
					break;
				case IDC_HISTORY:
					SendMessage(hMainWnd, WM_COMMAND, ID_TOPIC_HISTORY, 0);
					break;
				case IDC_NEXT:
					SendMessage(hMainWnd, WM_COMMAND, ID_TOPIC_NEXT, 0);
					break;
				case IDC_PREV:
					SendMessage(hMainWnd, WM_COMMAND, ID_TOPIC_PREV, 0);
					break;
				case IDC_SEARCH:
					SendMessage(hMainWnd, WM_COMMAND, ID_EDIT_SEARCH, 0);
					return(FALSE);
				}
			/*
			 * After a button is processed, put the focus back on the main
			 * window so that keystrokes will all be routed there.
			 */
			SetFocus(hMainWnd);
			break;
		case UWM_PANE_FOCUS:
			hActiveTopic = 0;
			break;
		case UWM_TOPIC_FOCUS:
			/* Clear selection from previous active Topic? */
			hWnd = (HWND)lParam;
			if (hActiveTopic && hActiveTopic != hWnd)
				{
				if (hWnd == lpTopic->hTopic && lpSecondary)
					Topic_ClearSelection(lpSecondary);
				else
					Topic_ClearSelection(lpTopic);
				}
			hActiveTopic = hWnd;
			break;
		}
	return (FALSE);
	}

/****************************************************************************
 **     FUNCTION: AboutDlgProc                                             **
 **     PURPOSE: About box handling                                        **
 **     COMMENTS:                                                          **
 ****************************************************************************/
BOOL WINAPI AboutDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
	UINT cmd;
	switch (message) 
		{
		case WM_INITDIALOG:
			SetFocus(hDlg);
			break;

		case WM_CLOSE:
			EndDialog(hDlg, FALSE);
			return(TRUE);
			break;

		case WM_COMMAND:
			cmd = GET_WM_COMMAND_ID(wParam, lParam);
			switch (cmd)
				{
				case IDOK:
					EndDialog(hDlg, TRUE);
					return(TRUE);
				}
			break;
		}
	return (FALSE);
	}
     
/****************************************************************************
 **     FUNCTION: SearchDlgProc                                            **
 **     PURPOSE: Full Text Search                                          **
 **     COMMENTS:                                                          **
 ****************************************************************************/
BOOL WINAPI SearchDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
	int scope, status;
	char ** groupList;
	HCURSOR hcurOld;
	int andFlag;
	WORD err;
	static char szSearch[512] = {0};
	LPGROUPUI lpGroup;
	UINT	cmd;
	char buff[10];

	switch (message) 
		{
		case WM_INITDIALOG:
			/* initialize to All Topics radio button */
			SendDlgItemMessage( hDlg, IDC_ALL, BM_SETCHECK, 1, 0);
			SetFocus(GetDlgItem(hDlg, IDC_SEARCH));

			/* disable the OK button until there is a search string */
			EnableWindow(GetDlgItem(hDlg, IDOK), FALSE);
			
			/* disable the Stop Search button until we are actually searching */
 			EnableWindow(GetDlgItem(hDlg, IDC_STOPSEARCH), FALSE);

			/* load up the Groups list (initially disabled) */
			EnableWindow(GetDlgItem(hDlg, IDC_GROUPLIST), FALSE);
			EnableWindow(GetDlgItem(hDlg, IDC_GROUPLABEL), FALSE);
			lpGroup = lpGroups;
			while (lpGroup && lpGroup->title)
				{
				SendDlgItemMessage(hDlg, IDC_GROUPLIST, LB_ADDSTRING, 0, (LPARAM)(LPSTR)lpGroup->title);
				++lpGroup;
				}
			
			/* initialize to the previous search word */
			SendDlgItemMessage(hDlg, IDC_SEARCH, WM_SETTEXT, 0, (LPARAM)(LPSTR)szSearch);
#ifdef WIN32
			SendDlgItemMessage(hDlg, IDC_SEARCH, EM_SETSEL, 0, strlen(szSearch));
#else
			SendDlgItemMessage(hDlg, IDC_SEARCH, EM_SETSEL, 0, MAKELPARAM(0, strlen(szSearch)));
#endif //WIN32
			break;
		case WM_CLOSE:
			EndDialog(hDlg, FALSE);
			return(TRUE);
			break;

		case WM_COMMAND:
			cmd = GET_WM_COMMAND_ID(wParam, lParam);
			switch (cmd)
				{
				case IDOK:
				/* this variable allows the search to be aborted midway */
				gStopSearch = FALSE;

					/* set up for the search */
					groupList = 0;

					/* which radio button is down? */
					if (SendDlgItemMessage( hDlg, IDC_TOPIC, BM_GETCHECK, 0, 0))
						scope = SEARCH_TOPIC;
					if (SendDlgItemMessage( hDlg, IDC_GROUPS, BM_GETCHECK, 0, 0))
						{
						groupList = UI_GetGroups(hDlg);
						scope = SEARCH_GROUPS;
						}
					if (SendDlgItemMessage( hDlg, IDC_ALL, BM_GETCHECK, 0, 0))
						scope = SEARCH_ALL;

					/* is this an AND or an OR of the search words? */
					if (SendDlgItemMessage( hDlg, IDC_AND, BM_GETCHECK, 0, 0))
						andFlag = TRUE;
					else
						andFlag = FALSE;
						
					/* enable the Stop Search and disable the OK and Cancel buttons while searching */
           			EnableWindow(GetDlgItem(hDlg, IDOK), FALSE);
           			EnableWindow(GetDlgItem(hDlg, IDCANCEL), FALSE);
           			EnableWindow(GetDlgItem(hDlg, IDC_STOPSEARCH), TRUE);

					/* get the search string and submit the query */
					SendDlgItemMessage(hDlg, IDC_SEARCH, WM_GETTEXT, sizeof(szSearch), (LPARAM)(LPSTR)szSearch);
					err = MV_Search(Topic_ValidMV(lpTopic), scope, andFlag, groupList, szSearch, hDlg);
					if (groupList)
						GlobalFreePtr(groupList);
					groupList = 0;
					hcurOld = SetCursor(LoadCursor(0, IDC_WAIT));

					/* restore the button states */
           			EnableWindow(GetDlgItem(hDlg, IDOK), TRUE);
           			EnableWindow(GetDlgItem(hDlg, IDCANCEL), TRUE);
           			EnableWindow(GetDlgItem(hDlg, IDC_STOPSEARCH), FALSE);
					
					/* if any were found, put up the search results Dialog */
					if (err == 0)
						{
						/* make sure that the Search Results display is visible */
						if (!hSearch)
							SendMessage(hMainWnd, WM_COMMAND, ID_VIEW_SEARCHRESULTS, 0);
						MV_ShowSearchResults(Topic_ValidMV(lpTopic));
						SetFocus(GetDlgItem(hSearch, IDC_LIST1));
						}
					else if (err == ERR_FAILED)
						{
						ErrorMsg("No matches", "Search Results");
						SetFocus(GetDlgItem(hDlg, IDC_SEARCH));
						SetCursor(hcurOld);
						break;
						}
					else
						{
						/* use MediaView to get error string */
						MV_ErrorMessage(err, szSearch, sizeof(szSearch));
						ErrorMsg(szSearch, "Search Results");
						SetFocus(GetDlgItem(hDlg, IDC_SEARCH));
						SetCursor(hcurOld);
						szSearch[0] = 0;
						break;
						}

					/* show the highlights if the flag is set */
					if (showHits)
						{
						MV_UpdateSearchHits(GetPaneMV(lpTopic->hSR));
						MV_UpdateSearchHits(GetPaneMV(lpTopic->hNSR));
						Topic_Layout(lpTopic);
						}
					SetCursor(hcurOld);
					gStopSearch = FALSE;

					EndDialog(hDlg, TRUE);
					return(TRUE);

				case IDCANCEL:
					EndDialog(hDlg, FALSE);
					return(TRUE);
					break;
				case IDC_TOPIC:		/* current topic */
					EnableWindow(GetDlgItem(hDlg, IDC_GROUPLIST), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_GROUPLABEL), FALSE);
					break;
				case IDC_GROUPS:		/* topics in group */
					EnableWindow(GetDlgItem(hDlg, IDC_GROUPLIST), TRUE);
					EnableWindow(GetDlgItem(hDlg, IDC_GROUPLABEL), TRUE);
					break;
				case IDC_ALL:		/* all topics */
					EnableWindow(GetDlgItem(hDlg, IDC_GROUPLIST), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_GROUPLABEL), FALSE);
					break;
				case IDC_STOPSEARCH:
					/* non-zero value here stops the search */
					gStopSearch = TRUE;
					break;
				case IDC_SEARCH:
					/* enable the OK button when there are characters */
					if (GET_WM_COMMAND_CMD(wParam, lParam) == EN_CHANGE)
						{
						status = (SendDlgItemMessage(hDlg, IDC_SEARCH, WM_GETTEXT, (WPARAM)(sizeof(buff)), (long)(LPSTR)buff) != 0);
						EnableWindow(GetDlgItem(hDlg, IDOK), status);
						}
					break;
				}
			break;
		}
	return (FALSE);
	}


/****************************************************************************
 **     FUNCTION: BookmarkDlgProc                                          **
 **     PURPOSE:                                                           **
 **     COMMENTS:                                                          **
 ****************************************************************************/
BOOL WINAPI BookmarkDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
	int status;
	char buff[512];
	UINT	cmd;
	switch (message) 
		{
		case WM_INITDIALOG:
			/* don't enable the "Add" button unless there is text */
			EnableWindow(GetDlgItem(hDlg, IDC_ADD), FALSE);
			SetFocus(GetDlgItem(hDlg, IDC_BOOKMARK));

			/* display existing bookmarks and init controls */
			MV_ShowBookmarks(GetDlgItem(hDlg, IDC_LIST1), hMainWnd);
			UI_EnableBookmarkControls(hDlg);
			break;

		case WM_CLOSE:
			EndDialog(hDlg, FALSE);
			return(TRUE);
			break;

		case WM_COMMAND:
			cmd = GET_WM_COMMAND_ID(wParam, lParam);
			switch (cmd)
				{
				case IDC_BOOKMARK:
					/* "Add" enabled as soon as there is text */
					if (GET_WM_COMMAND_CMD(wParam, lParam) == EN_CHANGE)
						{
						status = (SendDlgItemMessage(hDlg, IDC_BOOKMARK, WM_GETTEXT, (WPARAM)(sizeof(buff)), (long)(LPSTR)buff) != 0);
						EnableWindow(GetDlgItem(hDlg, IDC_ADD), status);
						}
					break;
				case IDOK:
					EndDialog(hDlg, TRUE);
					return(TRUE);
				case IDC_ADD:
					SendDlgItemMessage(hDlg, IDC_BOOKMARK, WM_GETTEXT, (WPARAM)(sizeof(buff)), (long)(LPSTR)buff);
					if (!MV_AddBookmark(hMainWnd, Topic_ValidMV(lpTopic), buff))
						ErrorMsg("Too many bookmarks.", "Bookmarks");
					MV_ShowBookmarks(GetDlgItem(hDlg, IDC_LIST1), hMainWnd);
					UI_EnableBookmarkControls(hDlg);
					break;
				case IDC_DELETE:
					MV_DelBookmark(GetDlgItem(hDlg, IDC_LIST1), hMainWnd);
					MV_ShowBookmarks(GetDlgItem(hDlg, IDC_LIST1), hMainWnd);
					UI_EnableBookmarkControls(hDlg);
					break;
				case IDC_LIST1:
					if (GET_WM_COMMAND_CMD(wParam, lParam) == LBN_SELCHANGE)
						UI_EnableBookmarkControls(hDlg);
					break;
				}
			break;
		}
	return (FALSE);
	}

/****************************************************************************
 **     FUNCTION: UI_EnableBookmarkControls                                **
 **     PURPOSE: Enable/Disable buttons appropriately in the Bookmark      **
 **        dialog.                                                         **
 **     COMMENTS:                                                          **
 ****************************************************************************/
void UI_EnableBookmarkControls(HWND hDlg)
	{
	if (SendDlgItemMessage(hDlg, IDC_LIST1, LB_GETCOUNT, 0, 0) == 0 ||
			SendDlgItemMessage(hDlg, IDC_LIST1, LB_GETCURSEL, 0, 0) == LB_ERR)
		EnableWindow(GetDlgItem(hDlg, IDC_DELETE), FALSE);
	else
		EnableWindow(GetDlgItem(hDlg, IDC_DELETE), TRUE);
	}
	         
/****************************************************************************
 **     FUNCTION: MagnifyDlgProc                                           **
 **     PURPOSE: Set the magnification factor on text.                     **
 **     COMMENTS:                                                          **
 ****************************************************************************/
BOOL WINAPI MagnifyDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
	int mag;
	char buff[100];
	UINT cmd;
	UINT i;
	LPMV lpMV;

	/*******************************************************/
	/******** TUTORIAL: Insert Lesson 14 code here. ********/
	/*******************************************************/
	return (FALSE);
	}

/****************************************************************************
 **     FUNCTION: KerningDlgProc                                           **
 **     PURPOSE: Set the kerning boundry             .                     **
 **     COMMENTS:                                                          **
 ****************************************************************************/
BOOL WINAPI KerningDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
	char buff[100];
	UINT	cmd;
	UINT i;

	/*******************************************************/
	/******** TUTORIAL: Insert Lesson 14 code here. ********/
	/*******************************************************/
	return (FALSE);
	}
/****************************************************************************
 **     FUNCTION: IndexDlgProc                                             **
 **     PURPOSE:                                                           **
 **     COMMENTS:                                                          **
 ****************************************************************************/
BOOL WINAPI IndexDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
	char buff[512];
	UINT	cmd;
	int selNumber;

	switch (message) 
		{
		case WM_INITDIALOG:
			UI_KeywordGroups(GetDlgItem(hDlg, IDC_KEYINDEX));
			MV_Keywords(Topic_ValidMV(lpTopic), GetDlgItem(hDlg, IDC_LIST1), GetDlgItem(hDlg, IDC_KEYINDEX));
			EnableWindow(GetDlgItem(hDlg, IDOK), FALSE);
			SetFocus(GetDlgItem(hDlg, IDC_INDEX));
			break;

		case WM_CLOSE:
			EndDialog(hDlg, FALSE);
			return(TRUE);

		case WM_COMMAND:
			cmd = GET_WM_COMMAND_ID(wParam, lParam);
			switch (cmd)
				{
				case IDC_INDEX:
					if (GET_WM_COMMAND_CMD(wParam, lParam) == EN_CHANGE)
						{
						if (SendDlgItemMessage(hDlg, IDC_INDEX, WM_GETTEXT, (WPARAM)(sizeof(buff)), (long)(LPSTR)buff) != 0)
							{
							/* look up the partial word and complete it */
							MV_IndexSelect(GetDlgItem(hDlg, IDC_LIST1), buff);
							EnableWindow(GetDlgItem(hDlg, IDOK), TRUE);
							}
						}
					break;
				case IDC_LIST1:
					/* enable OK button if there is a selection */
					if (GET_WM_COMMAND_CMD(wParam, lParam) == LBN_SELCHANGE)
						EnableWindow(GetDlgItem(hDlg, IDOK), TRUE);

					/* set the edit control to the selected text */
					selNumber = (int)SendDlgItemMessage(hDlg, IDC_LIST1, LB_GETCURSEL, 0, 0);
					if (selNumber == -1)
						break;
					SendDlgItemMessage(hDlg, IDC_LIST1, LB_GETTEXT, selNumber, (LPARAM)buff);
					SendDlgItemMessage(hDlg, IDC_INDEX, WM_SETTEXT, 0, (LPARAM)buff);

					/* if it is not a double click, we are done */
					if (GET_WM_COMMAND_CMD(wParam, lParam) != LBN_DBLCLK)
						break;
					/* otherwise fall through */
				case IDOK:
					if (MV_IndexLookup(hMainWnd,
									GetDlgItem(hDlg, IDC_LIST1),
									hSearch,
									lpTopic) != 0)
						ErrorMsg("Keyword Index Lookup Failed", "Out of Memory");
					EndDialog(hDlg, TRUE);
					return(TRUE);
				case IDCANCEL:
					EndDialog(hDlg, FALSE);
					return(TRUE);
				case IDC_KEYINDEX:
					/* if the selection changed, load the list box with the keywords for this index group */
					if (GET_WM_COMMAND_CMD(wParam, lParam) == CBN_SELCHANGE)
						{
						/* load up the main list box with this Index Group's keywords */
						iIndex = MV_Keywords(Topic_ValidMV(lpTopic), GetDlgItem(hDlg, IDC_LIST1), GetDlgItem(hDlg, IDC_KEYINDEX));

						/* clear the edit control and disable the OK button until there is a keyword selection */
						EnableWindow(GetDlgItem(hDlg, IDOK), FALSE);
						SendDlgItemMessage(hDlg, IDC_INDEX, WM_SETTEXT, (WPARAM)(sizeof(buff)), (long)(LPSTR)"");
						}
					break;
				}
			break;
		}
	return (FALSE);
	}


/****************************************************************************
 **     FUNCTION: IndexResultsDlgProc                                      **
 **     PURPOSE: When a keyword search has more than one hit.              **
 **     COMMENTS:                                                          **
 ****************************************************************************/
BOOL WINAPI IndexResultsDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
	int cmd;

	switch (message) 
		{
		case WM_INITDIALOG:
			MV_LoadIndexHits(GetDlgItem(hDlg, IDC_LIST1), Topic_ValidMV(lpTopic));

			/* disable the GOTO button until a selection is made */
			EnableWindow(GetDlgItem(hDlg, IDC_GOTO), FALSE);
			break;

		case WM_CLOSE:
			EndDialog(hDlg, TRUE);
			break;

		case WM_COMMAND:
			cmd = GET_WM_COMMAND_ID(wParam, lParam);
			switch (cmd)
				{
				case IDC_LIST1:

					/* if there is a selection, enable the GOTO button */
					if (GET_WM_COMMAND_CMD(wParam, lParam) != LBN_SELCHANGE)
						EnableWindow(GetDlgItem(hDlg, IDC_GOTO), TRUE);

					/* if not Double click, get out */
					if (GET_WM_COMMAND_CMD(wParam, lParam) != LBN_DBLCLK)
						break;
					/* otherwise fall through */
				case IDC_GOTO:
					MV_GoToFromIndex(GetDlgItem(hDlg, IDC_LIST1), Topic_ValidMV(lpTopic));
					EndDialog(hDlg, TRUE);
					return(TRUE);

				case IDCANCEL:
					EndDialog(hDlg, FALSE);
					return(TRUE);
				}
			break;
		}
	return (FALSE);
	}

/****************************************************************************
 **     FUNCTION: HistoryDlgProc                                           **
 **     PURPOSE:                                                           **
 **     COMMENTS:                                                          **
 ****************************************************************************/
BOOL WINAPI HistoryDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
	UINT	cmd;
	VA va;
	HTITLE hTitle;
	long scroll;
	int index;

	switch (message) 
		{
		case WM_INITDIALOG:
			/* disable the GOTO button until a selection is made */
			EnableWindow(GetDlgItem(hDlg, IDOK), FALSE);

			/* load up the history list */
			MV_ShowHistory(Topic_ValidMV(lpTopic), GetDlgItem(hDlg, IDC_LIST1));
			SetFocus(hDlg);
			break;

		case WM_CLOSE:
			EndDialog(hDlg, FALSE);
			hHist = 0;
			return(TRUE);

		case WM_COMMAND:
			cmd = GET_WM_COMMAND_ID(wParam, lParam);
			switch (cmd)
				{
				case IDC_LIST1:
					/* now the GoTo button can be enabled */
					if (GET_WM_COMMAND_CMD(wParam, lParam) == LBN_SELCHANGE)
						{
						EnableWindow(GetDlgItem(hDlg, IDOK), TRUE);
						break;
						}

					/* otherwise only a double click is interesting */
					if (GET_WM_COMMAND_CMD(wParam, lParam) != LBN_DBLCLK)
						break;
					/* fall through on double click */
				case IDOK:
					/* items are displayed in newest first, saved in history as oldest first */
					index = (int)SendDlgItemMessage(hDlg, IDC_LIST1, LB_GETCOUNT, 0, 0) - 1 -
								(int)SendDlgItemMessage(hDlg, IDC_LIST1, LB_GETCURSEL, 0, 0);
					MV_GoToFromHistory(index, &va, &hTitle, &scroll);
					MV_AddToHistory(Topic_ValidMV(lpTopic));
					MV_ChangeAddress(hTitle, lpTopic, va, scroll);
					return(TRUE);

				case IDCANCEL:
					EndDialog(hDlg, TRUE);
					hHist = 0;
					return(TRUE);
				}
			break;
		}
	return (FALSE);
	}
/****************************************************************************
 **     FUNCTION: MarkDlgProc                                              **
 **     PURPOSE: Handle marking topics for printing.                       **
 **     COMMENTS:                                                          **
 ****************************************************************************/
BOOL WINAPI MarkDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
	static int markType;
	LPGROUPUI lpGroup;
	UINT	cmd;

	switch (message) 
		{
		case WM_INITDIALOG:
			/* display all currently marked topics/groups */
			MV_ShowMarkList(GetDlgItem(hDlg, IDC_MARKLIST));
			lpGroup = lpGroups;

			/* walk through the groups and add them to the Groups list box */
			while (lpGroup && lpGroup->title)
				{
				SendDlgItemMessage(hDlg, IDC_GROUPLIST, LB_ADDSTRING, 0, (LPARAM)(LPSTR)lpGroup->title);
				++lpGroup;
				}

			/* initialize the dialog to select the current topic for marking */
			SendDlgItemMessage( hDlg, IDC_TOPIC, BM_SETCHECK, 1, 0);
			SendMessage(hDlg, WM_COMMAND, IDC_TOPIC, 0);

			/* which buttons should be enabled? */
			UI_EnableMarkControls(hDlg);
			break;

		case WM_CLOSE:
			EndDialog(hDlg, FALSE);
			return(TRUE);

		case WM_COMMAND:
			cmd = GET_WM_COMMAND_ID(wParam, lParam);
			switch (cmd)
				{
				case IDC_GROUPS:
					markType = wParam;
					UI_EnableMarkControls(hDlg);
					break;

				case IDC_TOPIC:
					markType = wParam;
					EnableWindow(GetDlgItem(hDlg, IDC_GROUPLIST), FALSE);
					EnableWindow(GetDlgItem(hDlg, IDC_GROUPLABEL), FALSE);
					UI_EnableMarkControls(hDlg);
					break;

				case IDOK:
					EndDialog(hDlg, TRUE);
					return(TRUE);

				case IDC_ADD:
					MV_AddPrintMark(Topic_ValidMV(lpTopic),
										 GetDlgItem(hDlg, IDC_MARKLIST),
										 GetDlgItem(hDlg, IDC_GROUPLIST),
										 markType);

					/* now there is something in the Mark list */
					UI_EnableMarkControls(hDlg);
					break;

				case IDC_DELETE:
					MV_DelPrintMark(GetDlgItem(hDlg, IDC_MARKLIST));
					MV_ShowMarkList(GetDlgItem(hDlg, IDC_MARKLIST));

					/* which buttons should be enabled? */
					UI_EnableMarkControls(hDlg);
					break;

				case IDC_CLEAR:
					MV_FreePrintMarks();
					MV_ShowMarkList(GetDlgItem(hDlg, IDC_MARKLIST));

					/* There is nothing marked; disable the Delete and Clear buttons */
					UI_EnableMarkControls(hDlg);
					break;

				case IDC_MARKLIST:
				case IDC_GROUPLIST:
					/* list box selection: which buttons should be enabled? */
					if (GET_WM_COMMAND_CMD(wParam, lParam) == LBN_SELCHANGE)
						UI_EnableMarkControls(hDlg);
					break;

				}
			break;
			}
	return (FALSE);
	}

/****************************************************************************
 **     FUNCTION: UI_EnableMarkControls                                    **
 **     PURPOSE: Enables/Disables the MarkDlg buttons appropriately        **
 **     COMMENTS:                                                          **
 ****************************************************************************/
void UI_EnableMarkControls(HWND hDlg)
	{
	/* if there is nothing marked, disable the Delete and Clear buttons */
	if (SendDlgItemMessage(hDlg, IDC_MARKLIST, LB_GETCOUNT, 0, 0) == 0)
		UI_EnableControls(hDlg, FALSE, IDC_DELETE, IDC_CLEAR, 0);

	/* if there is nothing selected disable the DELETE */
	else if (SendDlgItemMessage(hDlg, IDC_MARKLIST, LB_GETCURSEL, 0, 0) == LB_ERR)
		{
		EnableWindow(GetDlgItem(hDlg, IDC_DELETE), FALSE);
		EnableWindow(GetDlgItem(hDlg, IDC_CLEAR), TRUE);
		}

	/* otherwise enable them both */
	else
		UI_EnableControls(hDlg, TRUE, IDC_DELETE, IDC_CLEAR, 0);

	/* now, if "Current Topic" radio button is selected, enable Add */
	if (SendDlgItemMessage( hDlg, IDC_TOPIC, BM_GETCHECK, 0, 0))
		{
		EnableWindow(GetDlgItem(hDlg, IDC_ADD), TRUE);
		EnableWindow(GetDlgItem(hDlg, IDC_GROUPLIST), FALSE);
		EnableWindow(GetDlgItem(hDlg, IDC_GROUPLABEL), FALSE);
		}

	/* otherwise (All Topics in Group), don't enable add unless a Group is selected */
	else if (SendDlgItemMessage(hDlg, IDC_GROUPLIST, LB_GETCURSEL, 0, 0) != LB_ERR)
		{
		EnableWindow(GetDlgItem(hDlg, IDC_ADD), TRUE);
		EnableWindow(GetDlgItem(hDlg, IDC_GROUPLIST), TRUE);
		EnableWindow(GetDlgItem(hDlg, IDC_GROUPLABEL), TRUE);
		}
	else
		{
		EnableWindow(GetDlgItem(hDlg, IDC_ADD), FALSE);
		EnableWindow(GetDlgItem(hDlg, IDC_GROUPLIST), TRUE);
		EnableWindow(GetDlgItem(hDlg, IDC_GROUPLABEL), TRUE);
		}

	SetFocus(hDlg);
	}
/****************************************************************************
 **     FUNCTION: SearchResultsDlgProc                                     **
 **     PURPOSE:                                                           **
 **     COMMENTS:                                                          **
 ****************************************************************************/
BOOL WINAPI SearchResultsDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
	static RECT oldRect;
	static int oldPos = FALSE;
	UINT	cmd;
	VA va;

	switch (message) 
		{
		case WM_INITDIALOG:
			/* if there is a saved position, use it */
			if (oldPos)
				MoveWindow(hDlg, oldRect.left,
									  oldRect.top,
									  oldRect.right - oldRect.left,
									  oldRect.bottom - oldRect.top,
									  TRUE);

			/* put up the search results list (contained in the Topic) */
			hSearch = hDlg;
			MV_ShowSearchResults(Topic_ValidMV(lpTopic));
			SetFocus(GetDlgItem(hDlg, IDC_LIST1));
			break;

		case WM_CLOSE:
			Close:
			CheckMenuItem(GetMenu(hMainWnd), ID_VIEW_SEARCHRESULTS, FALSE);
			GetWindowRect(hDlg, &oldRect);
			oldPos = TRUE;
			hSearch = 0;
			EndDialog(hDlg, TRUE);
			break;

		case WM_COMMAND:
			cmd = GET_WM_COMMAND_ID(wParam, lParam);
			switch (cmd)
				{
				case IDC_LIST1:
					if (GET_WM_COMMAND_CMD(wParam, lParam) != LBN_DBLCLK)
						break;
					/* otherwise fall through */
				case IDC_GOTO:
					MV_GoToFromSearchResults(GetDlgItem(hDlg, IDC_LIST1), Topic_ValidMV(lpTopic), &va);
					MV_AddToHistory(Topic_ValidMV(lpTopic));
					MV_ChangeAddress(0, lpTopic, va, 0);
					MV_ScrollToSearchHighlight(GetPaneMV(lpTopic->hSR), 0);
					return(TRUE);

				case IDCANCEL:
					goto Close;
				}
			break;
		}
	return (FALSE);
	}


/****************************************************************************
 **     FUNCTION: ErrorMsg                                                 **
 **     PURPOSE: show an error string dialog box                           **
 **     COMMENTS:                                                          **
 ****************************************************************************/
void ErrorMsg(LPSTR szText, LPSTR szCaption)
	{
	extern HWND hMainWnd;
	extern HWND hCaptureWnd;

	MessageBox (
					hMainWnd,
					szText,
					szCaption,
					MB_ICONASTERISK | MB_OK);

	/* MessageBox takes back the mouse. Reset the mouse capture. */
	if (hCaptureWnd)
		SetCapture(hCaptureWnd);
	}

/****************************************************************************
 **     FUNCTION: UI_EnableControls                                        **
 **     PURPOSE: Enable/Disable a set of controls                          **
 **     COMMENTS:                                                          **
 ****************************************************************************/
void UI_EnableControls(HWND hDlg, int status, ...)
	{
	va_list marker;
	int control;

	va_start(marker, status);
	while ((control = va_arg(marker, int)) != 0)
		EnableWindow(GetDlgItem(hDlg, control), status);

	va_end(marker);
	}

/****************************************************************************
 **     FUNCTION: UI_EnableMenuItems                                       **
 **     PURPOSE: Enable/Disable a set of menu items                        **
 **     COMMENTS:                                                          **
 ****************************************************************************/
void UI_EnableMenuItems(HWND hWnd, int * pItemList, int iCount, UINT uEnable)
	{
	HMENU hMenu = GetMenu(hWnd);
	while (iCount--)
		EnableMenuItem(hMenu, *pItemList++, uEnable);
	}

/****************************************************************************
 **     FUNCTION: UI_ToggleMenuItem                                        **
 **     PURPOSE: toggle the CHECKED/UNCHECKED state of the menu item       **
 **     COMMENTS:                                                          **
 ****************************************************************************/
int UI_ToggleMenuItem(HWND hWnd, int iMenuItem)
	{
	int state;

	state = GetMenuState(GetMenu(hWnd), iMenuItem, MF_BYCOMMAND);
	state = (state == MF_CHECKED ? MF_UNCHECKED : MF_CHECKED);
	CheckMenuItem(GetMenu(hWnd), iMenuItem, state);
	return(state);
	}

/****************************************************************************
 **     FUNCTION: UI_GetGroups                                             **
 **     PURPOSE: Create a group list from the list box selections          **
 **     COMMENTS:                                                          **
 **        The returned group list must be freed by the caller.            **
 ****************************************************************************/
char ** UI_GetGroups(HWND hDlg)
	{
	int i, j, num;
	char buff[512];
	char **lpList;
	int *lpSels;
	LPGROUPUI lpGroup;

	/* get the group list for the title */
	lpGroup = lpGroups;

	/* get the selection list  */
	num = (int)SendDlgItemMessage( hDlg, IDC_GROUPLIST, LB_GETSELCOUNT, 0, 0);
	lpSels = (int *)GlobalAllocPtr(GHND, num * sizeof(int));
	SendDlgItemMessage( hDlg, IDC_GROUPLIST, LB_GETSELITEMS, num, (LPARAM)lpSels);
	lpList = (char **)GlobalAllocPtr(GHND, (num+1) * sizeof(char *));

	/* for each one, look up the corresponding groupname */
	for (i = 0; i < num; ++i)
		{
		/* get the ith selected group title */
		SendDlgItemMessage(hDlg, IDC_GROUPLIST, LB_GETTEXT, lpSels[i], (LPARAM)buff);

		/* look up the associated group name */
		for (j = 0; lpGroup[j].title != 0; ++j)
			{
			if (strcmp(lpGroup[j].title, buff) == 0)
				break;
			}
		lpList[i] = lpGroup[j].name;
		}
	GlobalFreePtr(lpSels);
	lpSels = 0;
	
	/* terminate the list */
	lpList[num] = 0;
	return(lpList);
	}

/****************************************************************************
 **     FUNCTION: UI_FreeGroups                                            **
 **     PURPOSE: Free the group list                                       **
 **     COMMENTS:                                                          **
 ****************************************************************************/
LPGROUPUI UI_FreeGroups(LPGROUPUI lpGroup)
	{
	LPGROUPUI lpG = lpGroup;

	while (lpG && lpG->title)
		{
		GlobalFreePtr(lpG->name);
		if (lpG->title)
			GlobalFreePtr(lpG->title);
		++lpG;
		}
	GlobalFreePtr(lpGroup);
	return(0);
	}
	
/****************************************************************************
 **     FUNCTION: UI_UpdateEnable                                          **
 **     PURPOSE: Enable/disable things in the UI according to the status   **
 **     COMMENTS:                                                          **
 ****************************************************************************/
void UI_UpdateEnable(int statusP, int statusN, int statusC)
	{
	extern int iH;			/* history index */
	extern int iB;			/* history Back index */

	HMENU hMenu = GetMenu(hMainWnd);

	/* "Prev" button and menu item */
	EnableWindow(GetDlgItem(hFP, IDC_PREV), statusP);
	EnableMenuItem(hMenu, ID_TOPIC_PREV,
						statusP ? MF_ENABLED|MF_BYCOMMAND : MF_GRAYED|MF_BYCOMMAND);

	/* "Next" button and menu item */
	EnableWindow(GetDlgItem(hFP, IDC_NEXT), statusN);
	EnableMenuItem(hMenu, ID_TOPIC_NEXT,
						statusN ? MF_ENABLED|MF_BYCOMMAND : MF_GRAYED|MF_BYCOMMAND);


	/* "History" button and menu entry*/
	EnableWindow(GetDlgItem(hFP, IDC_HISTORY), iH != -1);
	EnableMenuItem(hMenu, ID_TOPIC_HISTORY,
									iH == -1 ?
											MF_GRAYED| MF_BYCOMMAND :
											MF_ENABLED| MF_BYCOMMAND);

	/* "Back" button and menu entry*/
	EnableWindow(GetDlgItem(hFP, IDC_BACK), iB != -1);
	EnableMenuItem(hMenu, ID_TOPIC_BACK,
									iB == -1 ?
											MF_GRAYED| MF_BYCOMMAND :
											MF_ENABLED| MF_BYCOMMAND);
	}

/****************************************************************************
 **     FUNCTION: UI_InitCursors                                           **
 **     PURPOSE: Load up the cursors                                       **
 **     COMMENTS:                                                          **
 ****************************************************************************/
void UI_InitCursors(HINSTANCE hInst)
	{
	curT = LoadCursor(hInst, "HAND_T");
	curE = LoadCursor(hInst, "HAND_E");
	curB = LoadCursor(hInst, "HAND_B");
	curU = LoadCursor(hInst, "HAND_U");
	curArrow = LoadCursor(0, IDC_ARROW);
	curIbeam = LoadCursor(0, IDC_IBEAM);
	curWait = LoadCursor(0, IDC_WAIT);
	}

/****************************************************************************
 **     FUNCTION: UI_DeleteCursors                                         **
 **     PURPOSE: Free up the cursors                                       **
 **     COMMENTS:                                                          **
 ****************************************************************************/
void UI_DeleteCursors()
	{                  
	if ( curT )
		DestroyCursor(curT);
	if ( curE )
		DestroyCursor(curE);
	if ( curB )
		DestroyCursor(curB);
	if ( curU )
		DestroyCursor(curU);
	curT = curE = curB = curU = 0;
	
	/* no need to destroy the standard windows cursors. */
	}

/****************************************************************************
 **     FUNCTION: UI_CloseTitles                                           **
 **     PURPOSE: Close all open titles.                                    **
 **     COMMENTS:                                                          **
 ****************************************************************************/
void UI_CloseTitles()
	{
	int i;  
	if (OpenTitles)
		{
		for (i = 0; i < numOpenTitles; ++i)
			MVTitleClose(OpenTitles[i]);
		GlobalFreePtr(OpenTitles);
		OpenTitles = 0;
		}
	}

/****************************************************************************
 **     FUNCTION: UI_ParseCommandLine                                      **
 **     PURPOSE: Put the command line in the same form as a C program.     **
 **     COMMENTS:                                                          **
 ****************************************************************************/
int UI_ParseCmdLine(LPSTR lpC, LPSTR *argv, int maxArg)
	{
	int i;

	for (i = 0; i < maxArg; ++i)
		{
		if (*lpC == 0)
			break;
		while (*lpC && isspace(*lpC))
			++lpC;
		argv[i] = lpC;
		while (*lpC && !isspace(*lpC))
			++lpC;
		if (*lpC != 0)
			*lpC++ = 0;
		}
	return(i);
	}

LPTOPIC UI_OpenFile(LPSTR szFileName, HINSTANCE hInstance, HTITLE hTitle, LPSTR szInitialTopic)
	{          
	char buff[_MAX_PATH];
	HMENU hMenu = GetMenu(hMainWnd);
	int statusP, statusN;
	LPMV lpMV;
	
	/* destroy the secondary window if it exists */
	if (lpSecondary)
		SendMessage(lpSecondary->hTopic, WM_SYSCOMMAND, SC_CLOSE, 0);

	/* "close" any title that is already open */
	if (lpTopic && lpTopic->isOpen)
		UI_CloseFile();

	/* Open the title ... use WAIT cursor in case there is a delay */
	if (hTitle == 0)
		{
		SetCursor(curWait);
		hTitle = hMVTitleOpenEx(szFileName, (FARPROC)MV_SearchInterrupt, NULL);
		if (hTitle == 0)
			{
			ErrorMsg(szFileName, "MVTitleOpenEx fails");
			SetCursor(curArrow);
			return(0);
			}
		}

	/* save the Handle so we can later close it */
	if (numOpenTitles == 0)
		OpenTitles = (HTITLE *)GlobalAllocPtr(GHND, sizeof(HTITLE));
	else
		OpenTitles = (HTITLE *)GlobalReAllocPtr(OpenTitles,(numOpenTitles+1)*sizeof(HTITLE), GHND);
	OpenTitles[numOpenTitles++] = hTitle;

	/* create the right hand window TOPIC */
	lpTopic = Topic_Create(hInstance, hTitle, hFP, WS_CHILD, szInitialTopic, MV_Hotspot);
	if (lpTopic == 0)
		{
		SetCursor(curArrow);
		return(0);
		}
	hActiveTopic = lpTopic->hTopic;
		
	/* update the two MVs with current global options */
	MV_UpdateOptions(GetPaneMV(lpTopic->hNSR));
	MV_UpdateOptions(GetPaneMV(lpTopic->hSR));

  	/* now create a Table of Contents Pane */
	hTOC = UI_CreateTOC(hInstance, hTitle);

	/* position the whole thing correctly in the face plate */
	UI_ResizeFacePlate(hMainWnd, hFP, (lpTopic?lpTopic->hTopic:0), hTOC);
	/* load up the Search Groups and Keyword Index Groups */
	MV_LoadGroups(Topic_ValidMV(lpTopic));
	MV_LoadKeyIndexes(Topic_ValidMV(lpTopic));
	iIndex = 0;
	/* set the title bar to the title */
	lMVTitleGetInfo(hTitle, TTLINF_TITLE, sizeof(buff)-1, (long)(LPSTR)buff);
	SetWindowText(hMainWnd, buff);

	/* update the menu/button bar status */
	lpMV = Topic_ValidMV(lpTopic);
	statusP = (addrMVGetPrev(lpMV) != addrNil);
	statusN = (addrMVGetNext(lpMV) != addrNil);
	UI_UpdateEnable(statusP, statusN, FALSE);

	SetCursor(curArrow);

	/* BUGBUG: map the DLL names for online or local DLLs */
	return(lpTopic);
	}

/****************************************************************************
 **     FUNCTION: UI_CreateTOC                                             **
 **     PURPOSE: Create the Table of Contents Pane.                        **
 **     COMMENTS:                                                          **
 **        The left side window will contain the Contents topic from the   **
 **        sample application. By setting its hotspot callback to          **
 **        MV_Hotspot all hotspot clicks in the TOC will only affect the   **
 **        right window.                                                   **
 ****************************************************************************/
 HWND UI_CreateTOC(HINSTANCE hInstance, HTITLE hTitle)
 	{
	HWND hWnd = 0;

	LPMV lpTocMV;
	VA va;
	ERR err;

	if ((hWnd = Pane_Create(hFP, 
							hInstance,
							hTitle,
							SR_PANE,
							P_NOHITS|P_NOSELECT,
							MV_Hotspot)) == 0)

		return(0);

	/* set up the initial position */
	lpTocMV = GetPaneMV(hWnd);
	va = vaMVGetContents(hTitle);
	if (va == vaNil || !fMVSetAddress(lpTocMV, va, SR_PANE, 0, (LPERR)&err))
		{
		DestroyWindow(hWnd);
  		return(0);
  		}

  	/* and display */
	Pane_Layout(hWnd, 0);
	ShowWindow(hWnd, SW_SHOW);
	return(hWnd);
	}

/****************************************************************************
 **     FUNCTION: UI_CloseFile                                             **
 **     PURPOSE: Close the open title.                                     **
 **     COMMENTS:                                                          **
 **       If the title is being "closed" in the middle of the application  **
 **       it is still in the history list, so only free the associated     **
 **       resources. 			   	                                       **
 ****************************************************************************/
 void UI_CloseFile()
 	{                   
	/* clear the display windows */
	if (lpTopic)
		Topic_Destroy(lpTopic);
	lpTopic = 0;
	if (hTOC)
		Pane_Destroy(hTOC);
	
	hTOC = 0;

	/* in memory group list */
	lpGroups = UI_FreeGroups(lpGroups);
	/* in memory keyword index list */
	lpKeyIndex = UI_FreeGroups(lpKeyIndex);
	
	/* miscellaneous cleanup */
	if (hWordWheel)
		MVWordWheelClose(hWordWheel);
	hWordWheel = 0;
	iIndex = 0;	
	if (hTopicList)
		MVTopicListDestroy(hTopicList);
	hTopicList = 0;
	}

/****************************************************************************
 **     FUNCTION: UI_KeywordGroups                                         **
 **     PURPOSE: Load up the COMBO box with Keyword groups                 **
 **     COMMENTS:                                                          **
 ****************************************************************************/
int UI_KeywordGroups(HWND hGroups)
	{
	LPGROUPUI lpK = lpKeyIndex;
	int i = 0;

	/* if there are no keyword groups, disable the COMBO box */
	if (!lpK || lpK->title == 0)
		{
		EnableWindow(hGroups, FALSE);
		return(0);
		}
		
	while (lpK && lpK->title)
		{
		SendMessage(hGroups, CB_ADDSTRING, 0, (LPARAM)lpK->title);
		/* Keyword group names are single letters */
		SendMessage(hGroups, CB_SETITEMDATA, i, (LPARAM) *lpK->name);
		++lpK;
		++i;
		}

	/* default to the previous selection (if any) */
	SendMessage(hGroups, CB_SETCURSEL, iIndex, 0);
	return(0);
	}

/****************************************************************************
 **     FUNCTION: UI_NextHighlight                                         **
 **     PURPOSE: Turn on the Next Highlight menu/button?                   **
 **     COMMENTS:                                                          **
 ****************************************************************************/
void UI_NextHighlight(BOOL state)
	{
	HMENU hMenu = GetMenu(hMainWnd);

	UI_EnableControls(hFP, state, IDC_NEXTHIGHLIGHT, 0);
	EnableMenuItem(hMenu, ID_VIEW_NEXTHIGHLIGHT, MF_BYCOMMAND | (state ? MF_ENABLED: MF_GRAYED));
	}

/*****************************************************************************
 **     FUNCTION: UI_BaggageTest											**
 **     PURPOSE: Test the Baggage APIs										**
 **     COMMENTS:															**
 **			The file "TEST.XYZ" was placed in baggage of the MYTHS.M14		**
 **			test title. This routine demonstrates the use of the Baggage	**
 **			APIs.															**
 *****************************************************************************/
BOOL UI_BaggageTest(HTITLE hTitle)
	{
	HMVFILE hf;
	HANDLE hMem;
	LPBYTE lpB;
	DWORD dw;
	char c;
	ERR err;

	/* 100th char in file is 99 */
	hMem = hMVBaggageGetFile(hTitle, "TEST.XYZ", TRUE, 0, 100, &dw, &err);
	if (hMem == 0)
		return(FALSE);
	lpB = GlobalLock(hMem);
	if (lpB[99] != 99)
		return(FALSE);
	GlobalFree(hMem);
	hMem = 0;
	
  	/* open, check that size is 100 */
  	hf = hMVBaggageOpen(hTitle, "test.xyz", TRUE, &err);
	if (hf == 0)
		return(FALSE);
 	if (lMVBaggageSize(hf, &err) != 100)
		return(FALSE);

	/* read value, seek to it */
 	lMVBaggageRead(hf, (LPBYTE)&dw, sizeof(DWORD), &err);
 	lMVBaggageSeek(hf, dw, 0, &err);

	/* read value, seek and read */
 	lMVBaggageRead(hf, (LPBYTE)&dw, sizeof(DWORD), &err);
	lMVBaggageSeekRead(hf, dw, 0, &c, 1, &err);
	if (c != 99)
		return(FALSE);

	/* close the Baggage file */
	MVBaggageClose(hf, &err);

	return(TRUE);
	}       
	


